@@ -22,6 +22,9 @@ at the top-level directory.
  *     September 18, 2018  version 6.0
  *     February 8, 2019    version 6.1.1
  *     November 12, 2019   version 6.2.0
+ *     October 23, 2020    version 6.4.0
+ *     May 12, 2021        version 7.0.0
+ *     October 5, 2021     version 7.1.0
  * </pre>
  */
 
@@ -45,9 +48,10 @@ at the top-level directory.
 #include <stdio.h>
 #include <limits.h>
 #include <string.h>
-//#include <stdatomic.h>
+// #include <stdatomic.h>
 #include <math.h>
 #include <stdint.h>
+//#include <malloc.h>  Sherry: not available on Mac OS
 // /* Following is for vtune */
 // #if 0
 // #include <ittnotify.h>
@@ -70,10 +74,10 @@ at the top-level directory.
  *   #endif
  * Versions 4.x and earlier do not include a #define'd version numbers.
  */
-#define SUPERLU_DIST_MAJOR_VERSION     6
-#define SUPERLU_DIST_MINOR_VERSION     2
+#define SUPERLU_DIST_MAJOR_VERSION     7
+#define SUPERLU_DIST_MINOR_VERSION     1
 #define SUPERLU_DIST_PATCH_VERSION     0
-#define SUPERLU_DIST_RELEASE_DATE      "November 12, 2019"
+#define SUPERLU_DIST_RELEASE_DATE      "October 5, 2021"
 
 #include "superlu_dist_config.h"
 /* Define my integer size int_t */
@@ -84,21 +88,59 @@ at the top-level directory.
 #elif defined (_LONGINT)
   typedef int64_t int_t;
   #define mpi_int_t   MPI_LONG_LONG_INT
-  #define IFMT "%ld"
+  #define IFMT "%lld"
 #else /* Default */
   typedef int int_t;
   #define mpi_int_t   MPI_INT
   #define IFMT "%8d"
 #endif
 
+//#ifdef __INTEL_COMPILER
+//#include "mkl.h"
+//#endif
+
+#if 0 // Sherry: the following does not work with gcc on Linux.
+#define  _mm_malloc(a,b) malloc(a)
+#define _mm_free(a) free(a)
+
+static __inline__ unsigned long long _rdtsc(void)
+{
+    unsigned long long int x;
+    //    __asm__ volatile (".byte 0x0f, 0x31" : "=A" (x));
+    x = 0;
+    return x;
+}
+#endif
+
+#ifdef HAVE_CUDA  
+#define GPU_ACC   // enable CUDA
+#include "cublas_utils.h"
+#endif
+
+/* MPI C complex datatype */
+#define SuperLU_MPI_COMPLEX         MPI_C_COMPLEX 
+#define SuperLU_MPI_DOUBLE_COMPLEX  MPI_C_DOUBLE_COMPLEX
+
+/* MPI_Datatype cannot be used in C typedef
+typedef MPI_C_COMPLEX         SuperLU_MPI_COMPLEX;
+typedef MPI_C_DOUBLE_COMPLEX  SuperLU_MPI_DOUBLE_COMPLEX;
+*/
+
 #include "superlu_FortranCInterface.h"
-//#include "Cnames.h"
 #include "superlu_FCnames.h"
 #include "superlu_enum_consts.h"
 #include "supermatrix.h"
 #include "util_dist.h"
 #include "psymbfact.h"
 
+#ifdef GPU_ACC
+#include <cuda.h>
+#endif
+
+
+#define MAX_SUPER_SIZE 512   /* Sherry: moved from superlu_gpu.cu */
+
+
 #define ISORT     /* NOTE: qsort() has bug on Mac */
 
 /*********************************************************************** 
@@ -163,7 +205,7 @@ at the top-level directory.
  * 0,1: for sending L to "right"                                             *
  * 2,3: for sending off-diagonal blocks of U "down"                          *
  * 4  : for sending the diagonal blcok down (in pxgstrf2)                    */
-#define SLU_MPI_TAG(id,num) ( (5*(num)+id) % tag_ub )
+//#define SLU_MPI_TAG(id,num) ( (5*(num)+id) % tag_ub )
 
     /* For numeric factorization. */
 #if 0
@@ -310,16 +352,40 @@ typedef struct {
     int Iam;              /* my process number */
 } superlu_scope_t;
 
-/*-- Process grid definition */
+/*-- 2D process grid definition */
 typedef struct {
     MPI_Comm comm;        /* MPI communicator */
     superlu_scope_t rscp; /* process scope in rowwise, horizontal directon */
     superlu_scope_t cscp; /* process scope in columnwise, vertical direction */
-    int iam;              /* my process number in this scope */
+    int iam;              /* my process number in this grid */
     int_t nprow;          /* number of process rows */
     int_t npcol;          /* number of process columns */
 } gridinfo_t;
 
+/*-- 3D process grid definition */
+typedef struct {
+    MPI_Comm comm;        /* MPI communicator */
+    superlu_scope_t rscp; /* row scope */
+    superlu_scope_t cscp; /* column scope */
+    superlu_scope_t zscp; /* scope in third dimension */
+    gridinfo_t grid2d;    /* for using 2D functions */
+    int iam;              /* my process number in this grid */
+    int_t nprow;          /* number of process rows */
+    int_t npcol;          /* number of process columns */
+    int_t npdep;          /* number of replication factor in Z-dimension  */
+    int rankorder;        /* = 0: Z-major ( default )
+			   *    e.g. 1x3x4 grid: layer0 layer1 layer2 layer3
+			   *                     0      3      6      9
+			   *                     1      4      7      10      
+			   *                     2      5      8      11
+			   * = 1: XY-major (need set env. var.: RANKORDER=XY)
+			   *    e.g. 1x3x4 grid: layer0 layer1 layer2 layer3
+			   *                     0      1      2      4
+			   *                     5      6      7      8
+			   *                     9      10     11     12
+			   */
+} gridinfo3d_t;
+
 
 /*
  *-- The structures are determined by SYMBFACT and used thereafter.
@@ -397,12 +463,12 @@ typedef struct {
     int_t     nzlmax;    /* current max size of lsub */
     int_t     nzumax;    /*    "    "    "      usub */
     LU_space_t MemModel; /* 0 - system malloc'd; 1 - user provided */
-    int_t     *llvl;     /* keep track of level in L for level-based ILU */
-    int_t     *ulvl;     /* keep track of level in U for level-based ILU */
+    //int_t     *llvl;     /* keep track of level in L for level-based ILU */
+    //int_t     *ulvl;     /* keep track of level in U for level-based ILU */
     int64_t nnzLU;   /* number of nonzeros in L+U*/
 } Glu_freeable_t;
 
-
+#if 0 // Sherry: move to precision-dependent file
 /* 
  *-- The structure used to store matrix A of the linear system and
  *   several vectors describing the transformations done to matrix A.
@@ -447,6 +513,7 @@ typedef struct {
     int_t  *perm_r;
     int_t  *perm_c;
 } ScalePermstruct_t;
+#endif
 
 /*-- Data structure for redistribution of B and X --*/
 typedef struct {
@@ -634,20 +701,27 @@ typedef struct {
     yes_no_t      SolveInitialized;
     yes_no_t      RefineInitialized;
     yes_no_t      PrintStat;
-    int           nnzL, nnzU;      /* used to store nnzs for now       */
+    //int           nnzL, nnzU;      /* used to store nnzs for now       */
     int           num_lookaheads;  /* num of levels in look-ahead      */
     yes_no_t      lookahead_etree; /* use etree computed from the
 				      serial symbolic factorization */
     yes_no_t      SymPattern;      /* symmetric factorization          */
+    yes_no_t      Algo3d;          /* use 3D factorization/solve algorithms */
 } superlu_dist_options_t;
 
 typedef struct {
     float for_lu;
     float total;
-    int_t expansions;
+    int expansions;
     int64_t nnzL, nnzU;
 } superlu_dist_mem_usage_t;
 
+/*-- Auxiliary data type used in PxGSTRS/PxGSTRS1. */
+typedef struct {
+    int_t lbnum;  /* Row block number (local).      */
+    int_t indpos; /* Starting position in Uindex[]. */
+} Ucb_indptr_t;
+
 /* 
  *-- The new structures added in the hybrid CUDA + OpenMP + MPI code.
  */
@@ -656,13 +730,18 @@ typedef struct {
     int_t iukp;
     int_t jb;
     int_t full_u_cols;
-
+    int_t eo; /* order of elimination. For 3D algorithm */
+    int_t ncols;
+    int_t StCol;
 } Ublock_info_t;
 
 typedef struct {
     int_t lptr;
     int_t ib;
+    int_t eo; /* order of elimination, for 3D code */
+    int_t nrows;
     int_t FullRow;
+    int_t StRow;
 } Remain_info_t;
 
 typedef struct
@@ -679,6 +758,195 @@ struct superlu_pair
 
 /**--------**/
 
+/*==== For 3D code ====*/
+
+/* return the mpi_tag assuming 5 pairs of communications and MPI_TAG_UB >= 5 *
+ * for each supernodal column, the five communications are:                  *
+ * 0,1: for sending L to "right"                                             *
+ * 2,3: for sending off-diagonal blocks of U "down"                          *
+ * 4  : for sending the diagonal blcok down (in pxgstrf2)                    */
+// int tag_ub;
+// #define SLU_MPI_TAG(id,num) ( (5*(num)+id) % tag_ub )
+
+// #undef SLU_MPI_TAG
+/*defining my own MPI tags */
+/* return the mpi_tag assuming 5 pairs of communications and MPI_TAG_UB >= 5 *
+ * for each supernodal column, the five communications are:                  *
+ * 0,1: for sending L to "right"                                             *
+ * 2,3: for sending off-diagonal blocks of U "down"                          *
+ * 4  : for sending the diagonal blcok down (in pxgstrf2)                    *
+ * 5  : for sending the diagonal L block right () : added by piyush */
+#define SLU_MPI_TAG(id,num) ( (6*(num)+id) % tag_ub )
+
+/*structs for quick look up */
+typedef struct
+{
+    int_t luptrj;
+    int_t lptrj;
+    int_t lib;
+} local_l_blk_info_t;
+ 
+typedef struct
+{
+    int_t iuip;
+    int_t ruip;
+    int_t ljb;
+} local_u_blk_info_t;
+
+
+//global variable 
+extern double CPU_CLOCK_RATE;
+
+typedef struct
+{
+    int_t *perm_c_supno;
+    int_t *iperm_c_supno;
+}  perm_array_t;
+
+typedef struct
+{   
+    int_t* factored;
+    int_t* factored_D;
+    int_t* factored_L;
+    int_t* factored_U;
+    int_t* IrecvPlcd_D;
+    int_t* IbcastPanel_L;         /*I bcast and recv placed for the k-th L panel*/
+    int_t* IbcastPanel_U;         /*I bcast and recv placed for the k-th U panel*/
+    int_t* numChildLeft;            /*number of children left to be factored*/
+    int_t* gpuLUreduced;          /*New for GPU acceleration*/
+}factStat_t;
+
+typedef struct
+{
+    int_t next_col;
+    int_t next_k;
+    int_t kljb;
+    int_t kijb;
+    int_t copyL_kljb;
+    int_t copyU_kljb;
+    int_t l_copy_len;
+    int_t u_copy_len;
+    int_t *kindexL;
+    int_t *kindexU;
+    int_t mkrow;
+    int_t mkcol;
+    int_t ksup_size;
+} d2Hreduce_t;
+
+typedef struct{
+	int_t numChild;
+	int_t numDescendents;
+	int_t left;
+	int_t right;
+	int_t extra;
+	int_t* childrenList;
+	int_t depth; 		// distance from the top
+	double weight; 		// weight of the supernode
+	double iWeight; 	// weight of the whole subtree below
+	double scuWeight; 	// weight of schur complement update = max|n_k||L_k||U_k|
+} treeList_t;
+
+typedef struct 
+{
+	int_t numLvl;  // number of level in tree;
+	int_t* eTreeTopLims; // boundaries of each level  of size 
+	int_t* myIperm;		// Iperm for my tree size nsupers;
+	
+} treeTopoInfo_t;
+
+typedef struct 
+{
+	int_t* setree; 		// global supernodal elimination tree
+	int_t* numChildLeft;
+} gEtreeInfo_t; 
+
+typedef enum treePartStrat{
+	ND,				// nested dissection ordering or natural ordering
+	GD 				// greedy load balance stregy
+}treePartStrat;
+
+typedef struct
+{
+	/* data */
+	int_t nNodes; 			// total number of nodes
+	int_t* nodeList;		// list of nodes, should be in order of factorization
+#if 0 // Sherry: the following array is used on rForest_t. ???
+	int_t* treeHeads;
+#endif
+                            /*topological information about the tree*/
+	int_t numLvl;  			// number of Topological levels in the forest
+	int_t numTrees; 		// number of tree in the forest
+	treeTopoInfo_t  topoInfo; //
+#if 0  // Sherry fix: the following two structures are in treeTopoInfo_t. ???
+	int_t* eTreeTopLims; 	// boundaries of each level  of size 
+	int_t* myIperm;			// Iperm for my tree size nsupers;
+#endif
+
+	/*information about load balance*/
+	double weight;		// estimated cost 
+	double cost; 		// measured cost
+
+} sForest_t;
+
+typedef struct
+{
+    /* data */
+    MPI_Request* L_diag_blk_recv_req;
+    MPI_Request* L_diag_blk_send_req;
+    MPI_Request* U_diag_blk_recv_req;
+    MPI_Request* U_diag_blk_send_req;
+    MPI_Request* recv_req;
+    MPI_Request* recv_requ;
+    MPI_Request* send_req;
+    MPI_Request* send_requ;
+} commRequests_t;
+
+typedef struct
+{
+    int_t *iperm_c_supno;
+    int_t *iperm_u;
+    int_t *perm_u;
+    int *indirect;
+    int *indirect2;
+    
+} factNodelists_t;
+
+typedef struct
+{
+    int* msgcnt;
+    int* msgcntU;
+} msgs_t;
+
+typedef struct xtrsTimer_t
+{
+    double trsDataSendXY;
+    double trsDataSendZ;
+    double trsDataRecvXY;
+    double trsDataRecvZ;
+    double t_pdReDistribute_X_to_B;
+    double t_pdReDistribute_B_to_X;
+    double t_forwardSolve;
+    double tfs_compute;
+    double tfs_comm;
+    double t_backwardSolve;
+    double tbs_compute;
+    double tbs_comm;
+    double tbs_tree[2*MAX_3D_LEVEL];
+    double tfs_tree[2*MAX_3D_LEVEL];
+    
+    // counters for communication and computation volume 
+    
+    int_t trsMsgSentXY;
+    int_t trsMsgSentZ;
+    int_t trsMsgRecvXY;
+    int_t trsMsgRecvZ;
+    
+    double ppXmem;		// perprocess X-memory
+} xtrsTimer_t;
+
+/*==== For 3D code ====*/
+
+/*====================*/
 
 /***********************************************************************
  * Function prototypes
@@ -688,11 +956,14 @@ struct superlu_pair
 extern "C" {
 #endif
 
-extern void   set_default_options_dist(superlu_dist_options_t *);
-extern void   superlu_gridinit(MPI_Comm, int_t, int_t, gridinfo_t *);
-extern void   superlu_gridmap(MPI_Comm, int_t, int_t, int_t [], int_t,
-			      gridinfo_t *);
+extern void   superlu_gridinit(MPI_Comm, int, int, gridinfo_t *);
+extern void   superlu_gridmap(MPI_Comm, int, int, int [], int, gridinfo_t *);
 extern void   superlu_gridexit(gridinfo_t *);
+extern void   superlu_gridinit3d(MPI_Comm Bcomm,  int nprow, int npcol, int npdep,
+				 gridinfo3d_t *grid) ;
+extern void   superlu_gridexit3d(gridinfo3d_t *grid);
+
+extern void   set_default_options_dist(superlu_dist_options_t *);
 extern void   print_options_dist(superlu_dist_options_t *);
 extern void   print_sp_ienv_dist(superlu_dist_options_t *);
 extern void   Destroy_CompCol_Matrix_dist(SuperMatrix *);
@@ -732,7 +1003,6 @@ extern void    superlu_free_dist (void*);
 extern int_t   *intMalloc_dist (int_t);
 extern int_t   *intCalloc_dist (int_t);
 extern int_t   mc64id_dist(int_t *);
-extern int     c2cpp_GetAWPM(SuperMatrix *, gridinfo_t *, ScalePermstruct_t *);
 extern void  arrive_at_ublock (int_t, int_t *, int_t *, int_t *,
 			       int_t *, int_t *, int_t, int_t, 
 			       int_t *, int_t *, int_t *, gridinfo_t *);
@@ -742,12 +1012,9 @@ extern int_t estimate_bigu_size (int_t, int_t **, Glu_persist_t *,
 /* Auxiliary routines */
 extern double SuperLU_timer_ ();
 extern void   superlu_abort_and_exit_dist(char *);
-extern int_t  sp_ienv_dist (int_t);
+extern int    sp_ienv_dist (int);
 extern void   ifill_dist (int_t *, int_t, int_t);
 extern void   super_stats_dist (int_t, int_t *);
-extern void   ScalePermstructInit(const int_t, const int_t, 
-				   ScalePermstruct_t *);
-extern void   ScalePermstructFree(ScalePermstruct_t *);
 extern void  get_diag_procs(int_t, Glu_persist_t *, gridinfo_t *, int_t *,
 			    int_t **, int_t **);
 extern int_t QuerySpace_dist(int_t, int_t, Glu_freeable_t *, superlu_dist_mem_usage_t *);
@@ -792,6 +1059,18 @@ extern int_t psymbfact_LUXpand_RL
 extern int_t psymbfact_prLUXpand
 (int_t,  int_t, int, Llu_symbfact_t *, psymbfact_stat_t *);
 
+#ifdef ISORT
+extern void isort (int_t N, int_t *ARRAY1, int_t *ARRAY2);
+extern void isort1 (int_t N, int_t *ARRAY);
+#else
+int superlu_sort_perm (const void *arg1, const void *arg2)
+{
+    const int_t *val1 = (const int_t *) arg1;
+    const int_t *val2 = (const int_t *) arg2;
+    return (*val2 < *val1);
+}
+#endif
+
 #ifdef GPU_ACC   /* GPU related */
 extern void gemm_division_cpu_gpu (int *, int *, int *, int,
 				   int, int, int *, int);
@@ -799,12 +1078,16 @@ extern int_t get_cublas_nb ();
 extern int_t get_num_cuda_streams ();
 #endif
 
+extern double estimate_cpu_time(int m, int n , int k);
+
 extern int get_thread_per_process();
 extern int_t get_max_buffer_size ();
 extern int_t get_min (int_t *, int_t);
 extern int compare_pair (const void *, const void *);
 extern int_t static_partition (struct superlu_pair *, int_t, int_t *, int_t,
 			       int_t *, int_t *, int);
+extern int get_acc_offload();
+    
 
 /* Routines for debugging */
 extern void  print_panel_seg_dist(int_t, int_t, int_t, int_t, int_t *, int_t *);
@@ -817,7 +1100,6 @@ extern int   file_PrintInt10(FILE *, char *, int_t, int_t *);
 extern int   file_PrintInt32(FILE *, char *, int, int *);
 extern int   file_PrintLong10(FILE *, char *, int_t, int_t *);
 
-
 /* Routines for Async_tree communication*/
 
 #ifndef __SUPERLU_ASYNC_TREE /* allow multiple inclusions */
@@ -857,6 +1139,166 @@ extern int_t 	   	StdList_Size(StdList lst);
 yes_no_t 		StdList_Empty(StdList lst);
 
 
+/*==== For 3D code ====*/
+    
+extern void DistPrint(char* function_name,  double value, char* Units, gridinfo_t* grid);
+extern void DistPrint3D(char* function_name,  double value, char* Units, gridinfo3d_t* grid3d);
+extern void treeImbalance3D(gridinfo3d_t *grid3d, SCT_t* SCT);
+extern void SCT_printComm3D(gridinfo3d_t *grid3d, SCT_t* SCT);
+
+// permutation from superLU default
+extern int_t* getPerm_c_supno(int_t nsupers, superlu_dist_options_t *,
+			      int_t *etree, Glu_persist_t *Glu_persist, 
+			      int_t** Lrowind_bc_ptr, int_t** Ufstnz_br_ptr,
+			      gridinfo_t *);
+
+/* Manipulate counters */
+extern void SCT_init(SCT_t*);
+extern void SCT_print(gridinfo_t *grid, SCT_t* SCT);
+extern void SCT_print3D(gridinfo3d_t *grid3d, SCT_t* SCT);
+extern void SCT_free(SCT_t*);
+
+extern treeList_t* setree2list(int_t nsuper, int_t* setree );
+extern int  free_treelist(int_t nsuper, treeList_t* treeList);
+
+// int_t calcTreeWeight(int_t nsupers, treeList_t* treeList, int_t* xsup);
+extern int_t calcTreeWeight(int_t nsupers, int_t*setree, treeList_t* treeList, int_t* xsup);
+extern int_t getDescendList(int_t k, int_t*dlist,  treeList_t* treeList);
+extern int_t getCommonAncestorList(int_t k, int_t* alist,  int_t* seTree, treeList_t* treeList);
+extern int_t getCommonAncsCount(int_t k, treeList_t* treeList);
+extern int_t* getPermNodeList(int_t nnode, 	// number of nodes
+			      int_t* nlist, int_t* perm_c_sup,int_t* iperm_c_sup);
+extern int_t* getEtreeLB(int_t nnodes, int_t* perm_l, int_t* gTopOrder);
+extern int_t* getSubTreeRoots(int_t k, treeList_t* treeList);
+// int_t* treeList2perm(treeList_t* , ..);
+extern int_t* merg_perms(int_t nperms, int_t* nnodes, int_t** perms);
+// returns a concatenated permutation for three permutation arrays
+
+extern int_t* getGlobal_iperm(int_t nsupers, int_t nperms, int_t** perms,
+			      int_t* nnodes);
+extern int_t log2i(int_t index);
+extern int_t *supernodal_etree(int_t nsuper, int_t * etree, int_t* supno, int_t *xsup);
+extern int_t testSubtreeNodelist(int_t nsupers, int_t numList, int_t** nodeList, int_t* nodeCount);
+extern int_t testListPerm(int_t nodeCount, int_t* nodeList, int_t* permList, int_t* gTopLevel);
+
+/*takes supernodal elimination tree and for each 
+  supernode calculates "level" in elimination tree*/
+extern int_t* topological_ordering(int_t nsuper, int_t* setree);
+extern int_t* Etree_LevelBoundry(int_t* perm,int_t* tsort_etree, int_t nsuper);
+
+/*calculated boundries of the topological levels*/
+extern int_t* calculate_num_children(int_t nsuper, int_t* setree);
+extern void Print_EtreeLevelBoundry(int_t *Etree_LvlBdry, int_t max_level, int_t nsuper);
+extern void print_etree_leveled(int_t *setree,  int_t* tsort_etree, int_t nsuper);
+extern void print_etree(int_t *setree, int_t* iperm, int_t nsuper);
+extern int_t printFileList(char* sname, int_t nnodes, int_t*dlist, int_t*setree);
+int* getLastDepBtree( int_t nsupers, treeList_t* treeList);
+
+/*returns array R with of size maxLevel with either 0 or 1
+	R[i] = 1; then Tree[level-i] is set to zero= to only 
+	accumulate the results   */
+extern int_t* getReplicatedTrees( gridinfo3d_t* grid3d);
+
+/*returns indices in gNodeList of trees that belongs to my layer*/
+extern int_t* getGridTrees( gridinfo3d_t* grid3d);
+
+
+/*returns global nodelist*/
+extern int_t** getNodeList(int_t maxLvl, int_t* setree, int_t* nnodes,
+			int_t* treeHeads, treeList_t* treeList);
+
+/* calculate number of nodes in subtrees starting from treeHead[i]*/
+extern int_t* calcNumNodes(int_t maxLvl,  int_t* treeHeads, treeList_t* treeList);
+
+/*Returns list of (last) node of the trees */
+extern int_t* getTreeHeads(int_t maxLvl, int_t nsupers, treeList_t* treeList);
+
+extern int_t* getMyIperm(int_t nnodes, int_t nsupers, int_t* myPerm);
+
+extern int_t* getMyTopOrder(int_t nnodes, int_t* myPerm, int_t* myIperm, int_t* setree );
+
+extern int_t* getMyEtLims(int_t nnodes, int_t* myTopOrder);
+
+
+extern treeTopoInfo_t getMyTreeTopoInfo(int_t nnodes, int_t  nsupers,
+                                 int_t* myPerm,int_t* setree);
+
+extern sForest_t**  getNestDissForests( int_t maxLvl, int_t nsupers, int_t*setree, treeList_t* treeList);
+
+extern int_t** getTreePermForest( int_t* myTreeIdxs, int_t* myZeroTrIdxs,
+				  sForest_t*  sForests,
+				  int_t* perm_c_supno, int_t* iperm_c_supno,
+				  gridinfo3d_t* grid3d);
+extern int_t** getTreePermFr( int_t* myTreeIdxs,
+			      sForest_t**  sForests, gridinfo3d_t* grid3d);
+extern int_t* getMyNodeCountsFr(int_t maxLvl, int_t* myTreeIdxs,
+				sForest_t**  sForests);
+extern int_t** getNodeListFr(int_t maxLvl, sForest_t**  sForests);
+extern int_t*  getNodeCountsFr(int_t maxLvl, sForest_t**  sForests);
+// int_t* getNodeToForstMap(int_t nsupers, sForest_t**  sForests, gridinfo3d_t* grid3d);
+extern int_t* getIsNodeInMyGrid(int_t nsupers, int_t maxLvl, int_t* myNodeCount, int_t** treePerm);
+extern void printForestWeightCost(sForest_t**  sForests, SCT_t* SCT, gridinfo3d_t* grid3d);
+extern sForest_t**  getGreedyLoadBalForests( int_t maxLvl, int_t nsupers, int_t* setree, treeList_t* treeList);
+extern sForest_t**  getForests( int_t maxLvl, int_t nsupers, int_t*setree, treeList_t* treeList);
+
+    /* from trfAux.h */
+extern int_t getBigUSize(int_t nsupers, gridinfo_t *grid, int_t **Lrowind_bc_ptr);
+extern void getSCUweight(int_t nsupers, treeList_t* treeList, int_t* xsup,
+			 int_t** Lrowind_bc_ptr, int_t** Ufstnz_br_ptr,
+			 gridinfo3d_t * grid3d);
+extern int getNsupers(int n, Glu_persist_t *Glu_persist);
+extern int set_tag_ub();
+extern int getNumThreads(int);
+extern int_t num_full_cols_U(int_t kk, int_t **Ufstnz_br_ptr, int_t *xsup,
+			     gridinfo_t *, int_t *, int_t *);
+#if 0 // Sherry: conflicting with existing routine
+extern int_t estimate_bigu_size(int_t nsupers, int_t ldt, int_t**Ufstnz_br_ptr,
+				Glu_persist_t *, gridinfo_t*, int_t* perm_u);
+#endif
+extern int_t* getFactPerm(int_t);
+extern int_t* getFactIperm(int_t*, int_t);
+
+extern int_t initCommRequests(commRequests_t* comReqs, gridinfo_t * grid);
+extern int_t initFactStat(int_t nsupers, factStat_t* factStat);
+extern int   freeFactStat(factStat_t* factStat);
+extern int_t initFactNodelists(int_t, int_t, int_t, factNodelists_t*);
+extern int   freeFactNodelists(factNodelists_t* fNlists);
+extern int_t initMsgs(msgs_t* msgs);
+extern int_t getNumLookAhead(superlu_dist_options_t*);
+extern commRequests_t** initCommRequestsArr(int_t mxLeafNode, int_t ldt, gridinfo_t* grid);
+extern int   freeCommRequestsArr(int_t mxLeafNode, commRequests_t** comReqss);
+
+extern msgs_t** initMsgsArr(int_t numLA);
+extern int      freeMsgsArr(int_t numLA, msgs_t **msgss);
+
+extern int_t Trs2_InitUblock_info(int_t klst, int_t nb, Ublock_info_t *,
+                                  int_t *usub, Glu_persist_t *, SuperLUStat_t*);
+
+    /* from sec_structs.h */
+extern int Cmpfunc_R_info (const void * a, const void * b);
+extern int Cmpfunc_U_info (const void * a, const void * b);
+extern int sort_R_info( Remain_info_t* Remain_info, int n );
+extern int sort_U_info( Ublock_info_t* Ublock_info, int n );
+extern int sort_R_info_elm( Remain_info_t* Remain_info, int n );
+extern int sort_U_info_elm( Ublock_info_t* Ublock_info, int n );
+
+    /* from pdgstrs.h */
+extern void printTRStimer(xtrsTimer_t *xtrsTimer, gridinfo3d_t *grid3d);
+extern void initTRStimer(xtrsTimer_t *xtrsTimer, gridinfo_t *grid);
+
+    /* from p3dcomm.c */
+extern int_t** getTreePerm( int_t* myTreeIdxs, int_t* myZeroTrIdxs,
+                     int_t* nodeCount, int_t** nodeList,
+                     int_t* perm_c_supno, int_t* iperm_c_supno,
+                     gridinfo3d_t* grid3d);
+extern int_t* getMyNodeCounts(int_t maxLvl, int_t* myTreeIdxs, int_t* gNodeCount);
+extern int_t checkIntVector3d(int_t* vec, int_t len,  gridinfo3d_t* grid3d);
+extern int_t reduceStat(PhaseType PHASE, SuperLUStat_t *stat, gridinfo3d_t * grid3d);
+
+  extern int getnCudaStreams();
+  extern int get_mpi_process_per_gpu ();
+
+/*=====================*/
 
 #ifdef __cplusplus
   }
